--[[
## Title: FarmIt
## Notes: Custom loot tracker. By CHiLLZ [v1.3]
## SavedVariablesPerCharacter: FI_Config
##
## Author: CHiLLZ <chillz@xmission.com>
## License: Copyright 2006-2007, Chillz Media. All rights reserved.
]]--


-- SETTINGS --
local FI_Title = "FarmIt"
local FI_Version = 1.3;
local FI_MaxSlots = 12;
local FI_Debug = false;

local FI_Defaults = {
	["Version"] = FI_Version,
	["Options"] = {
		["NumSlots"] = 2,
		["TextAlign"] = "RIGHT",
		["ShowText"] = true,
	},
	["Items"] = {
		["FI_B1_S1"] = false,
		["FI_B1_S2"] = false,
	},
	["Display"] = {
		["FI"] = true,
		["FI_B1"] = true,
	},
}


-- LOADING --
function FI_OnLoad()
	FI:RegisterForDrag("LeftButton");
	FI:RegisterEvent("ADDON_LOADED");
	FI:RegisterEvent("UNIT_INVENTORY_CHANGED");
	FI:RegisterEvent("BAG_UPDATE");
	FI:RegisterEvent("BANKFRAME_OPENED");
	FI:RegisterEvent("BANKFRAME_CLOSED");
	
	SlashCmdList["farmit"] = FI_Command;
	SLASH_farmit1 = "/farmit";
end

function FI_OnShow()
	
end

function FI_OnEvent()
	if ((event == "ADDON_LOADED") and (arg1 == FI_Title)) then
		FI_Loader();
	end
	
	if (event == "UNIT_INVENTORY_CHANGED" or "BAG_UPDATE" or "BANKFRAME_OPENED") then
		FI_UpdateCounts();
	end
	
	if (event == "BANKFRAME_CLOSED") then
		FI_UpdateCounts();
	end
end

function FI_Loader()
	-- reset the data file?
	if (not FI_Config) then
		FI_Config = FI_Defaults;
		DEFAULT_CHAT_FRAME:AddMessage( "FarmIt v"..FI_Version.." initialized!  By CHiLLZ" );
	elseif (not FI_Config["Version"]) then
		FI_Config = FI_Defaults;
		DEFAULT_CHAT_FRAME:AddMessage( "FarmIt v"..FI_Version.." upgraded!  By CHiLLZ" );
	elseif (FI_Version > FI_Config["Version"]) then
		FI_Config = FI_Defaults;
		DEFAULT_CHAT_FRAME:AddMessage( "FarmIt v"..FI_Version.." upgraded.  By CHiLLZ" );
	else
		DEFAULT_CHAT_FRAME:AddMessage( "FarmIt v"..FI_Version.." loaded.  By CHiLLZ" );
	end
	DEFAULT_CHAT_FRAME:AddMessage( "    For a list of commands, type:    /farmit help" );
	
	-- start the mod
	FI_Core();
end

function FI_Core()
	-- create bar frame
	bar = "FI_B1";
	FI_CreateBar(bar, FI, FI);
	FI_B1_Anchor_Label:SetText(FI_Title);
	
	-- create slot frames
	for s=1,FI_MaxSlots do
		local slot = "_S"..s;
		
		-- dynamic anchoring
		if (s > 1) then
			local prev = "_S"..s-1;
			fAnchor = bar..prev;
		else
			fAnchor = bar;
		end
		
		-- create slot frame
		local fName = bar..slot;
		FI_CreateSlot(fName, FI_B1, fAnchor);
		
		-- load tracked items
		FI_SetSlot(fName);
		
		-- apply text settings
		FI_TextAlign(fName,  FI_Config["Options"]["TextAlign"]);
	end
	
	-- set visibility
	FI_TextVis(bar);
	FI_SetBar(bar);
	FI_SetVis("FI");
end

function FI_Reset()
	-- move to default location
	FI:ClearAllPoints();
	FI:SetPoint("LEFT", "UIParent", "LEFT", 0, 0);
	PlaySound("igCharacterInfoClose");
	
	-- copy default options over saved vars
	FI_Config = FI_Defaults;
	getglobal("FI_B1_Anchor_Label"):SetText(FI_Title);
	
	-- loop through slots
	local bar = "FI_B1";
	for s=1,FI_MaxSlots do
		local slot = "_S"..s;
		local fName = bar..slot;
		-- reset graphics
		FI_ClearSlot(fName);
		FI_TextAlign(fName, FI_Defaults["Options"]["TextAlign"]);
	end
	FI_TextVis(bar);
	FI_SetBar(bar);
	FI_SetVis("FI");
	
	DEFAULT_CHAT_FRAME:AddMessage( "Farmit:  Mod reset." );
end

-- FUNCTIONS --
function FI_CreateBar( fName, fParent, fAnchor )
	local f = CreateFrame("Frame", fName, fParent, "FI_TPL_BAR");
	f:SetFrameStrata("BACKGROUND");
	f:SetWidth(38);
	f:SetHeight(36);
	
	f:SetPoint("BOTTOM", fAnchor, "BOTTOM", 0,0);
end

function FI_CreateSlot( fName, fParent, fAnchor )
	local f = CreateFrame("Frame", fName, fParent, "FI_TPL_SLOT");
	f:SetFrameStrata("BACKGROUND");
	f:SetWidth(30);
	f:SetHeight(30);
	
	local t = f:CreateTexture(nil,"BACKGROUND");
	t:SetTexture("Interface\\PaperDoll\\UI-Backpack-EmptySlot");
	t:SetAllPoints(f);
	f.texture = t;
	
	f:SetPoint("BOTTOM", fAnchor, "TOP", 0,4);
end

function FI_Click( slot )
	if CursorHasItem() then
		local cursorType, itemID, cursorLink = GetCursorInfo();
		ClearCursor();
		FI_Config["Items"][slot] = itemID;
		FI_SetSlot(slot);
	elseif (arg1 == "RightButton") then
		FI_ClearSlot(slot);
		PlaySound("gsTitleOptionOK");
	end
end

function FI_SetSlot( slot )
	local itemID = FI_Config["Items"][slot];
	
	if (itemID) then
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemID);
		local itemCount = FI_Count(itemID);
		
		getglobal(slot.."_Icon"):SetTexture(itemTexture);
		getglobal(slot.."_Count"):SetText(itemCount);
		getglobal(slot.."_Text"):SetText(itemName);
		
		if (FI_Debug) then 
			DEFAULT_CHAT_FRAME:AddMessage( "Farmit:  "..slot.." set to "..itemLink );
		end
	else
		FI_ClearSlot(slot);
	end
end

function FI_ClearSlot( slot )
	if (FI_Config["Items"][slot]) then
		FI_Config["Items"][slot] = false;
	end
	
	getglobal(slot.."_Icon"):SetTexture("Interface\\PaperDoll\\UI-Backpack-EmptySlot");
	getglobal(slot.."_Count"):SetText("0");
	getglobal(slot.."_Text"):SetText("");
end

function FI_UpdateCounts()
	for s=1,FI_Config["Options"]["NumSlots"] do
		local slot = "FI_B1_S"..s;
		local itemID = FI_Config["Items"][slot];
		
		if (itemID) then
			local itemCount = FI_Count(itemID);
			getglobal(slot.."_Count"):SetText(itemCount);
		end
	end
end

function FI_ChangeBar( action )
	local current = FI_Config["Options"]["NumSlots"];
	
	-- change setting
	if (action == "+") then
		if (current < FI_MaxSlots) then
			FI_Config["Options"]["NumSlots"] = current+1;
		end
	elseif (action == "-") then
		if (current > 1) then
			FI_Config["Options"]["NumSlots"] = current-1;
		end
	end
	
	-- apply the change
	PlaySound("gsTitleOptionOK");
	FI_SetBar("FI_B1");
	FI_UpdateCounts();
end

function FI_SetBar( bar )
	for s=1,FI_MaxSlots do
		local slot = "_S"..s;
		local fName = bar..slot;
		getglobal(fName):Hide();
	end
	
	for s=1,FI_Config["Options"]["NumSlots"] do
		local slot = "_S"..s;
		local fName = bar..slot;
		getglobal(fName):Show();
	end
end

function FI_ToggleText( bar )
	-- flip the setting
	if (FI_Config["Options"]["ShowText"]) then
		FI_Config["Options"]["ShowText"] = false;
	else
		FI_Config["Options"]["ShowText"] = true;
	end
	-- apply the change
	FI_TextVis(bar);
	
	DEFAULT_CHAT_FRAME:AddMessage( "FarmIt:  ShowText = "..FI_Boolean(FI_Config["Options"]["ShowText"]) );
end

function FI_TextVis( bar )
	for s=1,FI_MaxSlots do
		local slot = bar.."_S"..s;
		if ( FI_Config["Options"]["ShowText"] ) then
			getglobal(slot.."_Text"):Show();
		else
			getglobal(slot.."_Text"):Hide();
		end
	end
end

function FI_ToggleAlign( bar )
	-- change the setting
	if (FI_Config["Options"]["TextAlign"] == "LEFT") then
		FI_Config["Options"]["TextAlign"] = "RIGHT";
	else
		FI_Config["Options"]["TextAlign"] = "LEFT";
	end
	-- apply the change
	for s=1,FI_MaxSlots do
		local slot = bar.."_S"..s;
		FI_TextAlign(slot,  FI_Config["Options"]["TextAlign"]);
	end
	
	DEFAULT_CHAT_FRAME:AddMessage( "FarmIt:  TextAlign = "..FI_Config["Options"]["TextAlign"] );
end

function FI_TextAlign( slot, align )
	local fAnchor = slot.."_Icon";
	local y=0;
	
	if (align == "LEFT") then
		fPoint = "RIGHT";
		x = "-10";
	else
		fPoint = "LEFT";
		x = "10";
	end
	
	getglobal(slot.."_Text"):ClearAllPoints();
	getglobal(slot.."_Text"):SetPoint(fPoint, fAnchor, align, x,y);
end

function FI_SetVis( element )
	local display = FI_Config["Display"][element];
	
	if (display) then
		getglobal(element):Show();
	else
		getglobal(element):Hide();
	end
end

function FI_ToggleVis( element )
	-- change setting
	FI_Config["Display"][element] = FI_Toggle(FI_Config["Display"][element]);
	
	-- apply change
	FI_SetVis( element );
	PlaySound("gsTitleOptionOK");
end


-- CORE LOGIC --
function FI_Count( itemID, startBag, endBag )
	local itemTotal = 0;
	local searchString = "item:"..itemID..":";
	
	if (not startBag) then
		startBag = -2;
		endBag = 11;
	end
	
	for bag = startBag,endBag do
		local slots = GetContainerNumSlots(bag);
		if (slots) then
			for slot = 1,slots do
				local itemLink = GetContainerItemLink(bag, slot);
				if (itemLink) then
					if ( string.find(itemLink, searchString) ) then
						local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag,slot);
						itemTotal = itemTotal + itemCount;
						
						if (FI_Debug) then 
							DEFAULT_CHAT_FRAME:AddMessage("FarmIt:  "..itemLink.." found at "..bag..","..slot);
						end
					end
				end
			end
		end
	end
	
	return itemTotal;
end

function FI_Stacks( limit, amount )
	local num,stacks;
	
	if (amount > limit) then
		num = amount / limit;
		stacks = format("%.1f", num);
	else
		stacks = 1;
	end
	
	return stacks;
end

function FI_Boolean( value )
	local output;
	
	if (value) then
		output = "true";
	else
		output = "false";
	end
	
	return output;
end

function FI_Toggle( value )
	if (value) then
		value = false;
	else
		value = true;
	end
	return value;
end


-- SLASH COMMANDS --
function FI_Command( cmd )
	if (cmd == "") then
		FI_ToggleVis("FI");
		DEFAULT_CHAT_FRAME:AddMessage( "FarmIt:  Show Mod = "..FI_Boolean(FI_Config["Display"]["FI"]) );
	elseif (cmd == "reset") then
		FI_Reset();
	elseif (cmd == "help") then
		DEFAULT_CHAT_FRAME:AddMessage( 
			"FarmIt:  Available commands...\n"..
			"     /farmit help  --  Shows this help message.\n"..
			"     /farmit reset  --  Resets the mod to default settings.\n".. 
			"     /farmit  --  Hide/Show the entire mod." 
		);
	end
end


-- STATS --
function FI_Track( itemID )
	-- coming in future version
end
